/*************************************************************/
// Copyright (c) 2020 Oracle and/or its affiliates.
/*************************************************************/
//Place holder to keep dynamically populated image ocid from OCI-RM (GUI)
variable "sbc_image_ocid_dyn" {
  default = ""
}

locals {
  num_instances = var.high_availability == true ? 2 : 1
  hostnames     = ["${var.prefix}-SBC0", "${var.prefix}-SBC1"]
  sshPubKey     = var.ssh_public_key == "" ? ["", "", ""] : regex("^(.+?) (.+?)( .*)?$", var.ssh_public_key)

  sbc_image_ocid = var.sbc_image_ocid != "" ? var.sbc_image_ocid : var.sbc_image_ocid_dyn
 
  //Flex Shape
  is_flex_shape = var.instance_shape == "VM.Optimized3.Flex" ? true : false

  //Sibling Cores
  use_sibling_cores = var.use_sibling_cores == true ? "enabled" : "disabled"

  //subnet count
  wancom1_subnet_count = var.high_availability == false ? 0 : var.wancom1_subnet_id == "" ? 0 : 1
  wancom2_subnet_count = var.high_availability == false ? 0 : var.wancom2_subnet_id == "" ? 0 : 1
  s0p0_subnet_count    = var.s0p0_subnet_id == "" ? 0 : 1
  s1p0_subnet_count    = var.s1p0_subnet_id == "" ? 0 : 1
  s0p1_subnet_count    = var.s0p1_subnet_id == "" ? 0 : 1
  s1p1_subnet_count    = var.s1p1_subnet_id == "" ? 0 : 1

  //Subnet masks
  wancom1_netmask = local.wancom1_subnet_count == 0 ? "" : cidrnetmask(data.oci_core_subnet.data_wancom1_subnet[0].cidr_block)
  wancom2_netmask = local.wancom2_subnet_count == 0 ? "" : cidrnetmask(data.oci_core_subnet.data_wancom2_subnet[0].cidr_block)


  fault_domain1 =  var.high_availability == false ? var.fault_domain != "" ? var.fault_domain : data.oci_identity_fault_domains.sbc_fault_domains.fault_domains[0].name : data.oci_identity_fault_domains.sbc_fault_domains.fault_domains[0].name
  fault_domain2 = length(data.oci_identity_fault_domains.sbc_fault_domains.fault_domains) > 1 ? data.oci_identity_fault_domains.sbc_fault_domains.fault_domains[1].name : data.oci_identity_fault_domains.sbc_fault_domains.fault_domains[0].name
  fault_domains = [local.fault_domain1, local.fault_domain2]

  ha_base_config = ((var.wancom1_subnet_id != "") && (var.wancom2_subnet_id != "")) ? "base_ha.xml" : (var.wancom2_subnet_id != "") ? "base_ha_wancom2.xml" : "base_ha_wancom1.xml"
  ha_config_file = var.high_availability == false ? "" : local.ha_base_config


  entitlements_file = {
    Session_Border_Controller            = "entitlements/sbc.yaml"
    Enterprise_Session_Border_Controller = "entitlements/esbc.yaml"
    Peering_Session_Border_Controller    = "entitlements/peering_sbc.yaml"
    Session_Router_Session_Stateful      = "entitlements/sr_session_stateful.yaml"
    Session_Router_Transaction_Stateful  = "entitlements/sr_transaction_stateful.yaml"
    Subscriber_Aware_Load_Balancer       = "entitlements/slb.yaml"
  }

  entitlements = yamldecode(file(local.entitlements_file[var.product_type]))
  mape         = tomap(local.entitlements.parameters.entitlements)
  map1         = { for key, value in local.mape : "entitlements.${key}" => value }

  partitions = {
    "partitions.app" = "20%"
    "partitions.sys" = "80%"
    "format.disk"    = "true"
  }

  map2 = var.create_disk_partitions == true ? local.partitions : {}

  map3 = merge(local.map1, local.map2)

  //vnic count
  wancom1_vnic_count = var.wancom1_subnet_id == "" ? 0 : var.high_availability == false ? 0 : 2
  wancom2_vnic_count = var.wancom2_subnet_id == "" ? 0 : var.high_availability == false ? 0 : 2
  s0p0_vnic_count    = var.s0p0_subnet_id == "" ? 0 : var.high_availability == false ? 1 : 2
  s1p0_vnic_count    = var.s1p0_subnet_id == "" ? 0 : var.high_availability == false ? 1 : 2
  s0p1_vnic_count    = var.s0p1_subnet_id == "" ? 0 : var.high_availability == false ? 1 : 2
  s1p1_vnic_count    = var.s1p1_subnet_id == "" ? 0 : var.high_availability == false ? 1 : 2

  ////////////////////
  //OIC Resource Manager related
  //List of Licenses, virtual IPs presented as comma seperated string

  licenses = split(",", var.licenses)

  s0p0_virtual_ips = var.s0p0_virtual_ips == "" ? [] : split(",", var.s0p0_virtual_ips)
  s1p0_virtual_ips = var.s1p0_virtual_ips == "" ? [] : split(",", var.s1p0_virtual_ips)
  s0p1_virtual_ips = var.s0p1_virtual_ips == "" ? [] : split(",", var.s0p1_virtual_ips)
  s1p1_virtual_ips = var.s1p1_virtual_ips == "" ? [] : split(",", var.s1p1_virtual_ips)
  ////////////////////

  license_keys = { for index, value in local.licenses : "license.key${index + 1}" => trimspace(value) }
  map_final    = merge(local.map3, local.license_keys)

  //VIP count
  s0p0_vip_count = local.s0p0_vnic_count != 2 ? 0 : length(local.s0p0_virtual_ips) == 0 ? var.s0p0_num_vips : length(local.s0p0_virtual_ips)
  s1p0_vip_count = local.s1p0_vnic_count != 2 ? 0 : length(local.s1p0_virtual_ips) == 0 ? var.s1p0_num_vips : length(local.s1p0_virtual_ips)
  s0p1_vip_count = local.s0p1_vnic_count != 2 ? 0 : length(local.s0p1_virtual_ips) == 0 ? var.s0p1_num_vips : length(local.s0p1_virtual_ips)
  s1p1_vip_count = local.s1p1_vnic_count != 2 ? 0 : length(local.s1p1_virtual_ips) == 0 ? var.s1p1_num_vips : length(local.s1p1_virtual_ips)

  ##  Create single reserved public IP for each media vNIC  ##
  s0p0_public_ip_count = var.s0p0_assign_public_ip == false ? 0 : 1
  s1p0_public_ip_count = var.s1p0_assign_public_ip == false ? 0 : 1
  s0p1_public_ip_count = var.s0p1_assign_public_ip == false ? 0 : 1
  s1p1_public_ip_count = var.s1p1_assign_public_ip == false ? 0 : 1

  metadata = [for i in range(local.num_instances) : jsonencode({ "meta" = merge(local.map_final, local.sbc_mactab[i]) })]

  base_template_vars = {
    "baseConfig"          = file("xmlconfig/base_config.xml")
    "restInterfaceConfig" = var.enable_rest_interface == false ? "" : file("xmlconfig/rest_interface_config.xml")
    "sshKeyConfig"        = var.ssh_public_key == "" ? "" : file("xmlconfig/ssh_config.xml")
    "redundancyConfig"    = var.high_availability == false ? "" : file("xmlconfig/${local.ha_config_file}")
  }

  sbc_mactab = [for i in range(local.num_instances) : {
    "bootparams.hostname" = local.hostnames[i]

    "mactab.wancom0" = data.oci_core_vnic.sbc_vnic_wancom0[i].mac_address
    "mactab.wancom1" = var.high_availability == false ? "" : var.wancom1_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_wancom1[i].mac_address
    "mactab.wancom2" = var.high_availability == false ? "" : var.wancom2_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_wancom2[i].mac_address

    "mactab.s0p0" = var.s0p0_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_s0p0[i].mac_address
    "mactab.s1p0" = var.s1p0_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_s1p0[i].mac_address
    "mactab.s0p1" = var.s0p1_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_s0p1[i].mac_address
    "mactab.s1p1" = var.s1p1_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_s1p1[i].mac_address

    "cores.forwarding"  = tostring(var.forwarding_cores)
    "cores.dos"         = tostring(var.dos_cores)
    "cores.transcoding" = tostring(var.transcoding_cores)
    "cores.useSibling"  = local.use_sibling_cores

    "passwords.userPass"  = var.user_pass
    "passwords.adminPass" = var.admin_pass

    #experimental
    //"bootparams.flags"   = "0x0010"
    "bootparams.console" = "ttyS0"
    }
  ]

  template_vars = [for i in range(local.num_instances) :
    {
      hostname               = var.high_availability == false ? local.hostnames[0] : join("-", local.hostnames)
      forwardingCores        = var.forwarding_cores
      dosCores               = var.dos_cores
      transcodingCores       = var.transcoding_cores
      useSiblingCoreDatapath = local.use_sibling_cores
      snmpCommunityName      = var.snmp_community_name
      snmpIpAddress          = var.snmp_ip_address
      ntpServer1             = var.ntp_server1
      ntpServer2             = var.ntp_server2
      sshPubKey              = local.sshPubKey[1]
      keySize                = ""
      fingerPrint            = ""
      fingerPrintRaw         = ""


      "primaryTargetName"   = local.hostnames[0]
      "secondaryTargetName" = local.hostnames[1]

      "primaryWancom1Netmask"   = local.wancom1_netmask
      "primaryWancom2Netmask"   = local.wancom2_netmask
      "secondaryWancom2Netmask" = local.wancom1_netmask
      "secondaryWancom2Netmask" = local.wancom2_netmask

      "primaryWancom1Ip"   = var.high_availability == false ? "" : var.wancom1_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_wancom1[0].private_ip_address
      "primaryWancom2Ip"   = var.high_availability == false ? "" : var.wancom2_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_wancom2[0].private_ip_address
      "secondaryWancom1Ip" = var.high_availability == false ? "" : var.wancom1_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_wancom1[1].private_ip_address
      "secondaryWancom2Ip" = var.high_availability == false ? "" : var.wancom2_subnet_id == "" ? "" : data.oci_core_vnic.sbc_vnic_wancom2[1].private_ip_address
    }
  ]
}

resource "null_resource" "input_validate" {
  count = local.validate
}

data "oci_core_subnet" "data_wancom0_subnet" {

  #Required
  subnet_id = var.wancom0_subnet_id
}

data "oci_core_subnet" "data_wancom1_subnet" {
  count = local.wancom1_subnet_count

  #Required
  subnet_id = var.wancom1_subnet_id
}

data "oci_core_subnet" "data_wancom2_subnet" {
  count = local.wancom2_subnet_count

  #Required
  subnet_id = var.wancom2_subnet_id
}


data "oci_identity_fault_domains" "sbc_fault_domains" {
  #Required
  availability_domain = var.availability_domain
  compartment_id      = var.compartment_ocid
}

data "template_file" "sbc_userdata" {
  count    = var.high_availability == false ? 1 : 2
  template = templatefile("xmlconfig/template_config.xml", local.base_template_vars)
  vars     = local.template_vars[count.index]
}


data "oci_objectstorage_namespace" "ns" {
  #Optional
  compartment_id = var.compartment_ocid
}

resource "oci_objectstorage_preauthrequest" "par_metadata" {
  count = var.high_availability == false ? 1 : 2

  namespace    = data.oci_objectstorage_namespace.ns.namespace
  bucket       = var.bucket_name
  object_name  = "${var.prefix}-metadata-object${count.index}"
  name         = "${var.prefix}-par_metadata-${count.index}"
  access_type  = "ObjectReadWrite" // ObjectRead, ObjectWrite, ObjectReadWrite, AnyObjectWrite
  time_expires = timeadd(timestamp(), "90m")
}

resource "oci_objectstorage_preauthrequest" "par_userdata" {
  count = var.high_availability == false ? 1 : 2

  namespace    = data.oci_objectstorage_namespace.ns.namespace
  bucket       = var.bucket_name
  object_name  = "${var.prefix}-userdata-object${count.index}"
  name         = "${var.prefix}-par_userdata-${count.index}"
  access_type  = "ObjectRead" // ObjectRead, ObjectWrite, ObjectReadWrite, AnyObjectWrite
  time_expires = timeadd(timestamp(), "90m")
}

resource "oci_objectstorage_object" "metadata" {
  count = var.high_availability == false ? 1 : 2

  namespace    = data.oci_objectstorage_namespace.ns.namespace
  bucket       = var.bucket_name
  object       = oci_objectstorage_preauthrequest.par_metadata[count.index].object
  content_type = "text"
  content      = var.apply_base_config == false ? "" : local.metadata[count.index]
}

resource "oci_objectstorage_object" "userdata" {
  count = var.high_availability == false ? 1 : 2

  namespace    = data.oci_objectstorage_namespace.ns.namespace
  bucket       = var.bucket_name
  object       = oci_objectstorage_preauthrequest.par_userdata[count.index].object
  content_type = "text"
  content      = var.apply_base_config == false ? "" : data.template_file.sbc_userdata[count.index].rendered
}


resource "oci_core_instance_console_connection" "sbc_instance_console_connection" {
  count = var.ssh_public_key == "" ? 0 : var.high_availability == false ? 1 : 2

  #Required
  instance_id = oci_core_instance.sbc_instance[count.index].id
  public_key  = var.ssh_public_key
}

resource "oci_core_instance" "sbc_instance" {
  count = var.high_availability == false ? 1 : 2

  availability_domain = var.availability_domain
  fault_domain        = local.fault_domains[count.index]
  compartment_id      = var.compartment_ocid
  display_name        = local.hostnames[count.index]
  shape               = var.instance_shape

  create_vnic_details {
    subnet_id        = var.wancom0_subnet_id
    display_name     = "${var.prefix}-vnic${count.index}-wancom0"
    assign_public_ip = ! data.oci_core_subnet.data_wancom0_subnet.prohibit_public_ip_on_vnic
    private_ip       = count.index == 0 ? var.wancom0_primary_node_ip : var.wancom0_secondary_node_ip
  }

  source_details {
    source_type = "image"
    source_id   = local.sbc_image_ocid

    # Apply this to set the size of the boot volume that's created for this instance.
    # Otherwise, the default boot volume size of the image is used.
    # This should only be specified when source_type is set to "image".
    boot_volume_size_in_gbs = var.boot_volume_size_in_gbs
  }

  dynamic "shape_config" {
    for_each = local.is_flex_shape ? [1] : []
    content {
      memory_in_gbs = var.memory_in_gbs
      ocpus         = var.ocpus
    }
  }

  metadata = {
    wait_time    = "300" //5 minutes
    par_metadata = "https://objectstorage.${var.region}.oraclecloud.com${oci_objectstorage_preauthrequest.par_metadata[count.index].access_uri}"
    par_userdata = "https://objectstorage.${var.region}.oraclecloud.com${oci_objectstorage_preauthrequest.par_userdata[count.index].access_uri}"
  }

  timeouts {
    create = "60m"
    delete = "30m"
  }
}

resource "oci_core_vnic_attachment" "sbc_s0p0" {
  count      = local.s0p0_vnic_count
  depends_on = [oci_core_vnic_attachment.sbc_wancom1, oci_core_vnic_attachment.sbc_wancom2] //serialize vnic creations

  #Required
  create_vnic_details {
    #Optional
    assign_public_ip = false
    display_name     = "${var.prefix}-vnic${count.index}-s0p0"
    private_ip       = count.index == 0 ? var.s0p0_primary_node_ip : var.s0p0_secondary_node_ip
    subnet_id        = var.s0p0_subnet_id
  }

  timeouts {
    create = "15m"
    delete = "15m"
  }

  instance_id = oci_core_instance.sbc_instance[count.index].id
}

resource "oci_core_private_ip" "sbc_s0p0_vips" {
  count = local.s0p0_vip_count

  ip_address = length(local.s0p0_virtual_ips) == 0 ? "" : trimspace(local.s0p0_virtual_ips[count.index])
  vnic_id    = oci_core_vnic_attachment.sbc_s0p0[0].vnic_id

  timeouts {
    create = "10m"
    delete = "10m"
  }
}

resource "oci_core_public_ip" "sbc_s0p0_public_ip" {
  count = local.s0p0_public_ip_count

  compartment_id = var.compartment_ocid
  lifetime       = "RESERVED"

  private_ip_id = var.high_availability == false ? data.oci_core_vnic.sbc_vnic_s0p0[0].private_ip_address : oci_core_private_ip.sbc_s0p0_vips[0].id
}

resource "oci_core_vnic_attachment" "sbc_s1p0" {
  count      = local.s1p0_vnic_count
  depends_on = [oci_core_vnic_attachment.sbc_s0p0] //serialize vnic creations

  #Required
  create_vnic_details {
    #Optional
    assign_public_ip = false
    display_name     = "${var.prefix}-vnic${count.index}-s1p0"
    private_ip       = count.index == 0 ? var.s1p0_primary_node_ip : var.s1p0_secondary_node_ip
    subnet_id        = var.s1p0_subnet_id
  }

  timeouts {
    create = "15m"
    delete = "15m"
  }

  instance_id = oci_core_instance.sbc_instance[count.index].id
}

resource "oci_core_private_ip" "sbc_s1p0_vips" {
  count = local.s1p0_vip_count

  ip_address = length(local.s1p0_virtual_ips) == 0 ? "" : trimspace(local.s1p0_virtual_ips[count.index])
  vnic_id    = oci_core_vnic_attachment.sbc_s1p0[0].vnic_id

  timeouts {
    create = "10m"
    delete = "10m"
  }
}


resource "oci_core_public_ip" "sbc_s1p0_public_ip" {
  count = local.s1p0_public_ip_count

  compartment_id = var.compartment_ocid
  lifetime       = "RESERVED"

  private_ip_id = var.high_availability == false ? data.oci_core_vnic.sbc_vnic_s1p0[0].private_ip_address : oci_core_private_ip.sbc_s1p0_vips[0].id
}


resource "oci_core_vnic_attachment" "sbc_s0p1" {
  count      = local.s0p1_vnic_count
  depends_on = [oci_core_vnic_attachment.sbc_s0p0, oci_core_vnic_attachment.sbc_s1p0] //serialize vnic creations

  #Required
  create_vnic_details {
    #Optional
    assign_public_ip = false
    display_name     = "${var.prefix}-vnic${count.index}-s0p1"
    private_ip       = count.index == 0 ? var.s0p1_primary_node_ip : var.s0p1_secondary_node_ip
    subnet_id        = var.s0p1_subnet_id
  }

  timeouts {
    create = "15m"
    delete = "15m"
  }

  instance_id = oci_core_instance.sbc_instance[count.index].id
}

resource "oci_core_private_ip" "sbc_s0p1_vips" {
  count = local.s0p1_vip_count

  ip_address = length(local.s0p1_virtual_ips) == 0 ? "" : trimspace(local.s0p1_virtual_ips[count.index])
  vnic_id    = oci_core_vnic_attachment.sbc_s0p1[0].vnic_id

  timeouts {
    create = "10m"
    delete = "10m"
  }
}

resource "oci_core_public_ip" "sbc_s0p1_public_ip" {
  count = local.s0p1_public_ip_count

  compartment_id = var.compartment_ocid
  lifetime       = "RESERVED"

  private_ip_id = var.high_availability == false ? data.oci_core_vnic.sbc_vnic_s0p1[0].private_ip_address : oci_core_private_ip.sbc_s0p1_vips[0].id
}

resource "oci_core_vnic_attachment" "sbc_s1p1" {
  count      = local.s1p1_vnic_count
  depends_on = [oci_core_vnic_attachment.sbc_s0p0, oci_core_vnic_attachment.sbc_s0p1] //serialize vnic creations

  #Required
  create_vnic_details {
    #Optional
    assign_public_ip = false
    display_name     = "${var.prefix}-vnic${count.index}-s1p1"
    private_ip       = count.index == 0 ? var.s1p1_primary_node_ip : var.s1p1_secondary_node_ip
    subnet_id        = var.s1p1_subnet_id
  }

  timeouts {
    create = "15m"
    delete = "15m"
  }

  instance_id = oci_core_instance.sbc_instance[count.index].id
}

resource "oci_core_private_ip" "sbc_s1p1_vips" {
  count = local.s1p1_vip_count

  ip_address = length(local.s1p1_virtual_ips) == 0 ? "" : trimspace(local.s1p1_virtual_ips[count.index])
  vnic_id    = oci_core_vnic_attachment.sbc_s1p1[0].vnic_id

  timeouts {
    create = "10m"
    delete = "10m"
  }
}

resource "oci_core_public_ip" "sbc_s1p1_public_ip" {
  count = local.s1p1_public_ip_count

  compartment_id = var.compartment_ocid
  lifetime       = "RESERVED"

  private_ip_id = var.high_availability == false ? data.oci_core_vnic.sbc_vnic_s1p1[0].private_ip_address : oci_core_private_ip.sbc_s1p1_vips[0].id
}

resource "oci_core_vnic_attachment" "sbc_wancom1" {
  count = local.wancom1_vnic_count

  #Required
  create_vnic_details {
    #Optional
    assign_public_ip = false
    display_name     = "${var.prefix}-vnic${count.index}-wancom1"
    private_ip       = count.index == 0 ? var.wancom1_primary_node_ip : var.wancom1_secondary_node_ip
    subnet_id        = var.wancom1_subnet_id
  }

  timeouts {
    create = "15m"
    delete = "15m"
  }

  instance_id = oci_core_instance.sbc_instance[count.index].id
}

resource "oci_core_vnic_attachment" "sbc_wancom2" {
  count      = local.wancom2_vnic_count
  depends_on = [oci_core_vnic_attachment.sbc_wancom1] //serialize vnic creations

  #Required
  create_vnic_details {
    #Optional
    assign_public_ip = false
    display_name     = "${var.prefix}-vnic${count.index}-wancom2"
    private_ip       = count.index == 0 ? var.wancom2_primary_node_ip : var.wancom2_secondary_node_ip
    subnet_id        = var.wancom2_subnet_id
  }

  timeouts {
    create = "15m"
    delete = "15m"
  }

  instance_id = oci_core_instance.sbc_instance[count.index].id
}

data "oci_core_private_ips" "wancom0_ip" {
  count = var.high_availability == false ? 1 : 2

  ip_address = oci_core_instance.sbc_instance[count.index].private_ip
  subnet_id  = oci_core_instance.sbc_instance[count.index].subnet_id
}

data "oci_core_vnic" "sbc_vnic_wancom0" {
  count = var.high_availability == false ? 1 : 2

  #Required
  vnic_id = data.oci_core_private_ips.wancom0_ip[count.index].private_ips[0].vnic_id
}

data "oci_core_vnic" "sbc_vnic_s0p0" {
  count = local.s0p0_vnic_count

  #Required
  vnic_id = oci_core_vnic_attachment.sbc_s0p0[count.index].vnic_id
}

data "oci_core_vnic" "sbc_vnic_s1p0" {
  count = local.s1p0_vnic_count

  #Required
  vnic_id = oci_core_vnic_attachment.sbc_s1p0[count.index].vnic_id
}

data "oci_core_vnic" "sbc_vnic_s0p1" {
  count = local.s0p1_vnic_count

  #Required
  vnic_id = oci_core_vnic_attachment.sbc_s0p1[count.index].vnic_id
}

data "oci_core_vnic" "sbc_vnic_s1p1" {
  count = local.s1p1_vnic_count

  #Required
  vnic_id = oci_core_vnic_attachment.sbc_s1p1[count.index].vnic_id
}

data "oci_core_vnic" "sbc_vnic_wancom1" {
  count = local.wancom1_vnic_count

  #Required
  vnic_id = oci_core_vnic_attachment.sbc_wancom1[count.index].vnic_id
}

data "oci_core_vnic" "sbc_vnic_wancom2" {
  count = local.wancom2_vnic_count

  #Required
  vnic_id = oci_core_vnic_attachment.sbc_wancom2[count.index].vnic_id
}
